@@ -25,6 +25,7 @@
 
 #include "idorange.h"
 #include "idotypebuiltins.h"
+#include "config.h"
 
 struct _IdoRangePrivate
 {
@@ -40,6 +41,10 @@ static void ido_range_get_property   (GObject          *object,
                                       guint             prop_id,
                                       GValue           *value,
                                       GParamSpec       *pspec);
+#ifdef USE_GTK3
+static void ido_range_grab_notify    (GtkWidget        *widget,
+                                      gboolean          was_grabbed);
+#endif
 
 #define IDO_RANGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), IDO_TYPE_RANGE, IdoRangePrivate))
 
@@ -60,6 +65,10 @@ ido_range_class_init (IdoRangeClass *class)
   gobject_class->set_property = ido_range_set_property;
   gobject_class->get_property = ido_range_get_property;
 
+#ifdef USE_GTK3
+  widget_class->grab_notify = ido_range_grab_notify;
+#endif
+
   g_object_class_install_property (gobject_class,
                                    PROP_STYLE,
                                    g_param_spec_enum ("range-style",
@@ -130,6 +139,21 @@ ido_range_set_property (GObject      *object,
     }
 }
 
+#ifdef USE_GTK3
+static void
+ido_range_grab_notify (GtkWidget *widget, gboolean was_grabbed)
+{
+  /*
+   * FIXME: workaround for lp bug #865122.
+   * Without this handler, GtkRange will call remove_grab which results
+   * in an infinite loop of grab_notifies.
+   *
+   * The widget will still work properly, because grab-broken-event will get
+   * properly fired and internal state of GtkRange will be properly updated.
+   */
+}
+#endif
+
 static void
 ido_range_constructed (GObject *object)
 {
@@ -153,6 +177,7 @@ ido_range_constructed (GObject *object)
                             "knob-height", &height,
                             NULL);
 
+#ifndef USE_GTK3
       g_snprintf (buf, sizeof (buf),
                   "style \"ido-range\" {\n"
                   "  GtkRange::slider-width = %d\n"
@@ -160,6 +185,7 @@ ido_range_constructed (GObject *object)
                   "} widget \"*.idorange-%p\" style \"ido-range\"\n",
                   width, height, range);
       gtk_rc_parse_string (buf);
+#endif
     }
 
   gtk_range_set_slider_size_fixed (GTK_RANGE (range), TRUE);
@@ -179,7 +205,7 @@ ido_range_init (IdoRange *range)
  * Creates a new #IdoRange widget.
  **/
 GtkWidget *
-ido_range_new (GtkAdjustment *adj,
+ido_range_new (GObject *adj,
                IdoRangeStyle  style)
 {
   g_return_val_if_fail (GTK_IS_ADJUSTMENT (adj), NULL);
